<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
	xmlns="http://jbg.sf.net/jbg"
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:seaf="http://seaf.sf.net/seaf"
	xmlns:per="http://seaf.sf.net/seaf-persistence"
	version="2.0">
<xsl:import href="functions/SeafPersistenceBusinessFunctions.xsl" />
<xsl:output method="xml" indent="yes" />

<xsl:param name="merged-file-name" />
<xsl:variable name="merged-persistence-file" select="document($merged-file-name)" />

<xsl:variable name="entities" select="$merged-persistence-file//per:persistence//per:entity" />
<xsl:variable name="relations" select="$merged-persistence-file//per:persistence//per:relation" />

<xsl:template match="/">
	<jbg xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
		xsi:schemaLocation="http://jbg.sf.net/jbg http://jbg.sf.net/xsd/1.0/jbg.xsd">
		<xsl:apply-templates select="//per:persistence/per:entity" />
	</jbg>
</xsl:template>

<xsl:template match="per:entity">
	<xsl:variable name="entity-name" select="@name" />
	<xsl:variable name="entity-class" select="seaf:type-class(@name)" />
	<xsl:variable name="type-label" select="per:persistent-object-type-label(.)" />
	<xsl:variable name="fields" select="per:entity-fields(.,$merged-persistence-file)" />
	<xsl:variable name="relation-details" 
		select="$relations/per:detail[@entity-name=$entity-name]" />
	<beans keep-collections-not-null="yes" final-accessors="no">
		<xsl:if test="not(@generate-entity='bean-only' or @generate-entity='entity-only')">
			<!-- The entity extending base bean -->
			<bean class="{per:entity-full-type(.)}" extends="{per:entity-bean-full-type(.)}"
				serializable-with-id="1">
				<xsl:apply-templates select="per:description" />
				<annotation>@javax.persistence.Entity</annotation>
				<annotation>@javax.persistence.Table(name = "<xsl:text />
					<xsl:value-of select="per:table-name($entity-class)" />")</annotation>
			</bean>
		</xsl:if>
		<xsl:if test="not(@generate-entity='entity-only')">
			<!-- The bean with properties -->
			<bean class="{per:entity-bean-full-type(.)}" 
				assume-instance-of="{per:entity-full-type(.)}"
				implements="{per:bean-interface(.)}" serializable-with-id="1">
				<description>The bean of entity <xsl:value-of select="@name"/>.</description>
				<annotation>@javax.persistence.Entity</annotation>
				<xsl:call-template name="entity-bean-content-template">
					<xsl:with-param name="fields" select="$fields" />
					<xsl:with-param name="relation-details" select="$relation-details" />
				</xsl:call-template>
			</bean>
		</xsl:if>
		<xsl:if test="@generate-entity='entity-only'">
			<!-- The entity with properties -->
			<bean class="{per:entity-full-type(.)}" implements="{per:bean-interface(.)}"
				serializable-with-id="1">
				<xsl:apply-templates select="per:description" />
				<annotation>@javax.persistence.Entity</annotation>
				<annotation>@javax.persistence.Table(name = "<xsl:text />
					<xsl:value-of select="per:table-name($entity-class)" />")</annotation>
				<xsl:call-template name="entity-bean-content-template">
					<xsl:with-param name="fields" select="$fields" />
					<xsl:with-param name="relation-details" select="$relation-details" />
				</xsl:call-template>
			</bean>
		</xsl:if>
	</beans>
	<beans chainable-setters="yes" keep-collections-not-null="yes" to-string-properties="all"
		final-accessors="no">
		<xsl:variable name="criteria-full-type" select="per:entity-criteria-full-type(.)" />
		<xsl:variable name="criteria-class" select="seaf:type-class($criteria-full-type)" />
		<!-- The criteria bean -->
		<bean class="{$criteria-full-type}" 
			implements="net.sf.seaf.persistence.CriteriaBean&lt;{$criteria-class}.Property>">
			<description>
				<xsl:text>The criteria of </xsl:text>
				<xsl:value-of select="$type-label" />
				<xsl:text> </xsl:text>
				<xsl:value-of select="@name"/>
				<xsl:text>.</xsl:text>
			</description>
			<xsl:apply-templates select="$fields[not(@status='version')]" mode="operator" />
			<property name="sortCriteria_" status="business-key">
				<sequence type="list">
					<bean-type type="PropertySort" />
				</sequence>
			</property>
			<additional-code>
				<xsl:value-of select="per:property-enum($fields)" />
				<xsl:value-of select="per:get-property-names-method()" />
				<xsl:value-of select="per:get-master-pk-criteria-method($relation-details)" />
				<xsl:value-of select="per:sort-criteria($criteria-class)" />
			</additional-code>
		</bean>
	</beans>
</xsl:template>

<xsl:template name="entity-bean-content-template">
<xsl:param name="fields" />
<xsl:param name="relation-details" />
	<xsl:variable name="entity-name" select="@name" />
	<!-- fields -->
	<xsl:apply-templates select="$fields[not(@status='criteria-only')]">
		<xsl:with-param name="entity-name" select="$entity-name" />
	</xsl:apply-templates>
	<!-- relations to masters -->
	<xsl:apply-templates select="$relation-details" />
	<!-- relations to details -->
	<xsl:apply-templates select="$relations/per:master[@entity-name=$entity-name]" />
	<!-- Delegate getters/setters to fulfill interfaces -->
	<xsl:if test="not($fields[@name='id']) 
			or (@code-table and not($fields[@name='code']))
			or (@code-table and not($fields[@name='label']))
			or (@code-table='code-based' and not($fields[@name='valid']))
			or (@code-table='time-bound' and not($fields[@name='validFrom']))
			or (@code-table='time-bound' and not($fields[@name='validUntil']))
			">
		<additional-code>
			<xsl:if test="not($fields[@name='id'])">
				<xsl:variable name="pk-field-name" select="per:persitent-object-pk-field(.)/@name" />
				<xsl:variable name="pk-type" select="per:persitent-object-pk-type(.)" />
				<xsl:value-of select="seaf:delegate-getter-setter('id',$pk-type,$pk-field-name)" />
			</xsl:if>
			<xsl:if test="@code-table and not($fields[@name='code'])">
				<xsl:variable name="field" select="per:field[@status='code-table-code']" />
				<xsl:value-of select="seaf:delegate-getter-setter('code','String',$field/@name)" />
			</xsl:if>
			<xsl:if test="@code-table and not($fields[@name='label'])">
				<xsl:variable name="field" select="per:field[@status='code-table-label']" />
				<xsl:value-of select="seaf:delegate-getter-setter('label','String',$field/@name)" />
			</xsl:if>
			<xsl:if test="@code-table='code-based' and not($fields[@name='valid'])">
				<xsl:variable name="field" select="per:field[@status='code-table-valid']" />
				<xsl:value-of select="seaf:delegate-getter-setter('valid','Boolean',$field/@name)" />
			</xsl:if>
			<xsl:if test="@code-table='time-bound' and not($fields[@name='validFrom'])">
				<xsl:variable name="field" select="per:field[@status='code-table-valid-from']" />
				<xsl:value-of select="seaf:delegate-getter-setter(
					'validFrom','java.util.Calendar',$field/@name)" />
			</xsl:if>
			<xsl:if test="@code-table='time-bound' and not($fields[@name='validUntil'])">
				<xsl:variable name="field" select="per:field[@status='code-table-valid-until']" />
				<xsl:value-of select="seaf:delegate-getter-setter(
					'validUntil','java.util.Calendar',$field/@name)" />
			</xsl:if>
		</additional-code>
	</xsl:if>
</xsl:template>

<xsl:template match="per:field">
<xsl:param name="entity-name" />
	<xsl:choose>
	<xsl:when test="@status='primary-key-assigned'">
		<property name="{@name}" status="business-key">
			<xsl:call-template name="property-content-template">
				<xsl:with-param name="entity-name" select="$entity-name" />
			</xsl:call-template>
		</property>
	</xsl:when>
	<xsl:when test="@status='primary-key-generated' or @status='transient' or @status='version'">
		<property name="{@name}" status="supplementary">
			<xsl:call-template name="property-content-template">
				<xsl:with-param name="entity-name" select="$entity-name" />
			</xsl:call-template>
		</property>
	</xsl:when>
	<xsl:otherwise>
		<property name="{@name}">
			<xsl:call-template name="property-content-template">
				<xsl:with-param name="entity-name" select="$entity-name" />
			</xsl:call-template>
		</property>
	</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="per:field" mode="operator">
<xsl:param name="entity-name" />
	<property name="{@name}" status="business-key">
		<xsl:call-template name="property-content-template">
			<xsl:with-param name="entity-name" select="$entity-name" />
			<xsl:with-param name="mode" select="'operator'" />
		</xsl:call-template>
	</property>
	<property name="{seaf:operator-field-name(@name)}" status="business-key">
		<description>Operator for field <xsl:value-of select="@name" />.</description>
		<simple-type type="enum" class="net.sf.seaf.common.Operator" />
	</property>
</xsl:template>

<xsl:template name="property-content-template">
<xsl:param name="entity-name" />
<xsl:param name="mode" />
	<xsl:apply-templates select="per:description" />
	<xsl:choose>
	<xsl:when test="per:enum">
		<bean-type type="{per:enum-ref-full-type(per:enum/@name,$merged-persistence-file)}"
			treat-as-simple="true" />
	</xsl:when>
	<xsl:when test="per:code-table">
		<bean-type type="{per:entity-ref-full-type(per:code-table/@name,$merged-persistence-file)}"
			treat-as-simple="true" />
	</xsl:when>
	<xsl:when test="per:binary">
		<sequence type="array">
			<simple-type type="byte" />
		</sequence>
	</xsl:when>
	<xsl:otherwise>
		<simple-type type="{per:field-simple-property-type(.)}" />
	</xsl:otherwise>
	</xsl:choose>
	<xsl:if test="not($mode='operator')">
		<xsl:if test="@status='primary-key-assigned' or @status='primary-key-generated'">
			<annotation>@javax.persistence.Id</annotation>
		</xsl:if>
		<xsl:if test="@status='primary-key-generated'">
			<annotation>@javax.persistence.GeneratedValue</annotation>
		</xsl:if>
		<xsl:if test="@status='version'">
			<annotation>@javax.persistence.Version</annotation>
		</xsl:if>
		<xsl:if test="per:enum or per:code-table">
			<annotation>@javax.persistence.ManyToOne</annotation>
		</xsl:if>
		<annotation>@javax.persistence.Column(name = "<xsl:text />
			<xsl:value-of select="per:column-name(@name)" />"<xsl:text />
			<xsl:if test="per:string">
				<xsl:text />, length = <xsl:value-of select="per:string/@length" />
			</xsl:if>
			<xsl:text />)</annotation>
	</xsl:if>
</xsl:template>

<xsl:template match="per:description">
	<description><xsl:value-of select="." /></description>
</xsl:template>

<xsl:template match="per:detail | per:master">
	<xsl:variable name="related-entity-name" select="per:related-entity-name(.,..)" />
	<xsl:variable name="related-entity" select="$entities[@name=$related-entity-name]" />
	<xsl:if test="count($related-entity) != 1">
		<xsl:message select="'Error locating related entity: exactly 1 expected'"/>
		<xsl:message select="concat(' reference to = ',$related-entity-name)" />
		<xsl:message select="concat('  - referenced entities found = ',count($related-entity))" />
	</xsl:if>
	<xsl:variable name="this-entity-name" select="@entity-name" />
	<xsl:variable name="this-entity" select="$entities[@name=$this-entity-name]" />
	<xsl:if test="count($this-entity) != 1">
		<xsl:message select="'Error locating entity: exactly 1 expected'"/>
		<xsl:message select="concat(' reference to = ',$this-entity-name)" />
		<xsl:message select="concat('  - referenced entities found = ',count($this-entity))" />
	</xsl:if>
	<property name="{per:relation-property-name(.,..)}">
		<description>
			<xsl:value-of select="concat('Relation to ',name(per:opposite-relation-side(.,..)),
				' entity ',$related-entity-name,'.')" />
		</description>
		<xsl:choose>
		<xsl:when test="per:is-collection(.,..)">
			<sequence type="collection">
				<xsl:call-template name="related-bean-type-template">
					<xsl:with-param name="related-entity" select="$related-entity" />
				</xsl:call-template>
			</sequence>
		</xsl:when>
		<xsl:otherwise>
			<xsl:call-template name="related-bean-type-template">
				<xsl:with-param name="related-entity" select="$related-entity" />
			</xsl:call-template>
		</xsl:otherwise>
		</xsl:choose>
		<xsl:if test="../@type='one-to-many' and name(.)='master'">
			<annotation>@javax.persistence.OneToMany(mappedBy = "<xsl:text />
				<xsl:value-of select="per:opposite-relation-property-name(.,..)"/>",
			cascade = javax.persistence.CascadeType.ALL, orphanRemoval = true)</annotation>
		</xsl:if>
		<xsl:if test="../@type='one-to-many' and name(.)='detail'">
			<annotation>@javax.persistence.ManyToOne</annotation>
		</xsl:if>
		<xsl:if test="../@type='one-to-one' and name(.)='master'">
			<annotation>@javax.persistence.OneToOne(mappedBy = "<xsl:text />
				<xsl:value-of select="per:opposite-relation-property-name(.,..)"/>",
			cascade = javax.persistence.CascadeType.ALL, orphanRemoval = true)</annotation>
		</xsl:if>
		<xsl:if test="../@type='one-to-one' and name(.)='detail'">
			<annotation>@javax.persistence.OneToOne</annotation>
		</xsl:if>
	</property>
</xsl:template>

<xsl:template name="related-bean-type-template">
<xsl:param name="related-entity" />
	<bean-type type="{per:entity-full-type($related-entity)}">
		<xsl:if test="not(per:is-opposite-collection(.,..))">
			<xsl:attribute name="reverse-property-name">
				<xsl:value-of select="per:opposite-relation-property-name(.,..)"/>
			</xsl:attribute>
		</xsl:if>
		<xsl:if test="local-name(.)='detail'">
			<xsl:attribute name="separate-reverse-setter">false</xsl:attribute>
		</xsl:if>
	</bean-type>
</xsl:template>

<xsl:function name="per:property-enum" as="xs:string">
	<xsl:param name="fields" as="node()*" />

	<xsl:variable name="result">
	<xsl:text />

	/**
	 * Names of properties.
	 */
	public enum Property {
		<xsl:for-each select="$fields[not(@status='transient' 
			or @status='criteria-only' or @status='version')]">
			<xsl:value-of select="@name" />
			<xsl:if test="not(position()=last())">, </xsl:if>
		</xsl:for-each>
	}<xsl:text />
	</xsl:variable>
	<xsl:value-of select="$result"/>
</xsl:function>

<xsl:function name="per:get-property-names-method" as="xs:string">
<xsl:text>

	public Iterable&lt;?> getPropertyNames() {
		return Arrays.asList(Property.values());
	}</xsl:text>
</xsl:function>

<xsl:function name="per:get-master-pk-criteria-method" as="xs:string">
	<xsl:param name="relation-details" as="node()*" />
	<xsl:variable name="result">

	public Iterable&lt;net.sf.seaf.persistence.MasterPkCriterion> getMasterPkCriteria() {
		return Arrays.asList(<xsl:text />
		<xsl:if test="count($relation-details)>1">
			<xsl:text>
			</xsl:text>
		</xsl:if>
		<xsl:for-each select="$relation-details">
			<xsl:variable name="entity-name" select="per:related-entity-name(.,..)" />
			<xsl:variable name="entity" select="$entities[@name=$entity-name]" />
			<xsl:variable name="pk-field" select="per:persitent-object-pk-field($entity)" />
			<xsl:variable name="criteria-field" select="concat(per:relation-property-name(.,..),'Id')" />

			<xsl:text />new net.sf.seaf.persistence.MasterPkCriterion("<xsl:text />
			<xsl:value-of select="$criteria-field" />", "<xsl:text />
			<xsl:value-of select="$pk-field/@name" />")<xsl:text />
			<xsl:if test="not(position()=last())">,
			</xsl:if>
		</xsl:for-each>
		<xsl:text />);
	}<xsl:text />
	</xsl:variable>
	<xsl:value-of select="$result" />
</xsl:function>

<xsl:function name="per:sort-criteria" as="xs:string">
	<xsl:param name="criteria-class" as="xs:string" />

	<xsl:variable name="result">
	<xsl:text />

	// sort criteria

	public static final class PropertySort extends net.sf.seaf.common.PropertySort&lt;Property> {
		public PropertySort() {
		}
		public PropertySort(Property property, net.sf.seaf.common.SortOrder order) {
			super(property, order);
		}
		public PropertySort(Property property) {
			super(property);
		}
		private static final long serialVersionUID = 1L;
	}

	/**
	 * Set the sort criteria to sort on the given property.
	 * 
	 * @param property
	 *            The property to sort on
	 * @param order
	 *            The order in which to sort
	 * @return this
	 */
	public <xsl:value-of select="$criteria-class" />
		<xsl:text /> setSort(Property property, net.sf.seaf.common.SortOrder order) {
		sortCriteria_.clear();
		return addSort(property, order);
	}

	/**
	 * Add sort on the given property to the sort criteria.
	 * 
	 * @param property
	 *            The property to sort on
	 * @param order
	 *            The order in which to sort
	 * @return this
	 */
	public <xsl:value-of select="$criteria-class" />
		<xsl:text /> addSort(Property property, net.sf.seaf.common.SortOrder order) {
		sortCriteria_.add(new PropertySort(property, order));
		return this;
	}<xsl:text />
	</xsl:variable>
	<xsl:value-of select="$result"/>
</xsl:function>

</xsl:stylesheet>
