package net.sf.seaf.persistence;

import java.util.Arrays;

import net.sf.seaf.util.HashCodeUtils;
import net.sf.seaf.util.ToStringUtils;

/**
 * A search criterion description on a master entity primary key.
 */
public class MasterPkCriterion {

	/**
	 * Name of the property on the criteria bean.
	 */
	private final String criteriaProperty;

	/**
	 * Name of the primary key property on the master entity bean.
	 */
	private final String masterPkProperty;

	/**
	 * Full constructor.
	 * 
	 * @param criteriaProperty
	 * @param masterPkProperty
	 */
	public MasterPkCriterion(String criteriaProperty, String masterPkProperty) {
		this.criteriaProperty = criteriaProperty;
		this.masterPkProperty = masterPkProperty;
	}

	/**
	 * Default getter for criteriaProperty.
	 * 
	 * @return The criteriaProperty
	 */
	public final String getCriteriaProperty() {
		return criteriaProperty;
	}

	/**
	 * Default getter for criteriaProperty.
	 * 
	 * @return The criteriaProperty
	 */
	public final String getMasterPkProperty() {
		return masterPkProperty;
	}

	// Java standard methods implementation

	/**
	 * Return the values of the business key properties of the bean.
	 * 
	 * @return Object[]
	 */
	private Object[] businessKeyValues() {
		return new Object[] { getCriteriaProperty(), getMasterPkProperty() };
	}

	@Override
	public int hashCode() {
		int superHashCode = 0;
		if (!MasterPkCriterion.class.getSuperclass().equals(Object.class))
			superHashCode = super.hashCode();
		return HashCodeUtils.hashCodeFor(superHashCode, businessKeyValues());
	}

	@Override
	public boolean equals(Object object) {
		if (this == object)
			return true;
		if (null == object)
			return false;
		if (!(getClass().equals(object.getClass())))
			return false;
		if (!MasterPkCriterion.class.getSuperclass().equals(Object.class)
				&& !super.equals(object))
			return false;
		final MasterPkCriterion that = (MasterPkCriterion) object;
		return Arrays
				.equals(this.businessKeyValues(), that.businessKeyValues());
	}

	@Override
	public String toString() {
		ToStringUtils u = new ToStringUtils(MasterPkCriterion.class,
				super.toString());
		u.addProperty("criteriaProperty", getCriteriaProperty());
		u.addProperty("masterPkProperty", getMasterPkProperty());
		return u.toString();
	}

}
